/**
 * @module dixim
 * @submodule autoDownload
 */
dixim.namespace('dixim.autoDownload');

/**
 * @module autoDownload
 * @submodule filter-edit-dialog-controller
 */

(function() {
var FilterEditDialogController;
var prototype;
var superConstructor;
var FilterEditDialogView = dixim.autoDownload.FilterEditDialogView;
var EventTarget = dixim.event.EventTarget;
var Event = dixim.event.Event;
var _ = dixim.ctt.ResourceBundle.get;
var ApiClientAutoDownload = dixim.common.ApiClientAutoDownload;
var CondDialogController = dixim.autoDownload.CondDialogController;
var Enum = dixim.ctt.util.Enum;
var ErrorModalDialogView = dixim.common.ErrorModalDialogView;
var Utils = dixim.ctt.util.Utils;

/**
 * @class FilterEditDialogController
 * @constructor
 * @param {Object} configs
 */
FilterEditDialogController = function(configs) {
    EventTarget.call(this);
    this._initialize(configs);
};

dixim.inherit(FilterEditDialogController, EventTarget);
dixim.autoDownload.FilterEditDialogController = FilterEditDialogController;
prototype = FilterEditDialogController.prototype;
superConstructor = FilterEditDialogController.superConstructor;

var SELECT_CONDITION = 'selectCondition';
var SETTING = 'setting';
var NO_COND = 'noCond';
var UPDATE = 'update';

var COND = Enum.fetch('COND');

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
    var len;

    configs = configs || {};

    configs.id = configs.id || '';
    this._id = [configs.id, 'filter-edit-dialog'].join('');

    this._model = new ApiClientAutoDownload();
    this._model.addEventListener('setFilter', this._successSetFilter, this);
    this._model.addEventListener('setFilterError', this._errorSetFilter, this);
    this._model.addEventListener('setCond', this._successChangeFilter, this);
    this._model.addEventListener('setCondError', this._errorChangeFilter, this);
    this._model.addEventListener('removeCond', this._successChangeFilter, this);
    this._model.addEventListener('removeCondError', this._errorChangeFilter, this);
    this._model.addEventListener('changeFilterName', this._successChangeFilter, this);
    this._model.addEventListener('changeFilterNameError', this._errorChangeFilter, this);
    this._model.addEventListener('changeFilterComb', this._successChangeFilter, this);
    this._model.addEventListener('changeFilterCombError', this._errorChangeFilter, this);

    this._view = new FilterEditDialogView(configs);
    this._view.addEventListener(SELECT_CONDITION, this._clickSelectCondition, this);
    this._view.addEventListener(SETTING, this._clickSetting, this);

    this._filterInfo = {
        title: '',
        combCond: 'AND',
        conds: {},
    };
    for (var key in COND) {
        this._filterInfo.conds[COND[key]] = [];
    }
    if (typeof configs.filterInfo !== 'undefined') {
        if (typeof configs.filterInfo.id !== 'undefined') this._filterInfo.id = configs.filterInfo.id;
        if (typeof configs.filterInfo.title !== 'undefined') this._filterInfo.title = configs.filterInfo.title;
        if (typeof configs.filterInfo.combCond !== 'undefined') this._filterInfo.combCond = configs.filterInfo.combCond;
    }
    if (typeof configs.conds !== 'undefined') {
        len = configs.conds.length;
        for (var i = 0; i < len; i += 1) {
            this._filterInfo.conds[configs.conds[i].name].push(configs.conds[i]);
        }
    }

    this._requestingCondInfo = {};
    this._isEdit = (this._filterInfo.title !== '');
    if (this._isEdit) {
        this._oldFilterInfo = $.extend(true, {}, this._filterInfo);
    }
};

/**
 * @method start
 */
prototype.start = function() {
    this._view.show();
    if (this._isEdit) {
        this._view.setFilterInfo(this._filterInfo);
    }
};

/**
 * @method stop
 */
prototype.stop = function() {
    this._view.hide();
};

prototype._delCondInFilterInfo = function(condId) {
    this._filterInfo.conds[condId] = [];
};

prototype._setFilter = function() {
    var query = {};

    query.filterName = this._filterInfo.title;
    query.combCond = this._filterInfo.combCond;

    this._view.startIndicator();
    this._model.setFilter(query);
};

prototype._successSetFilter = function(e) {
    var conds = this._extractCond(this._filterInfo);
    var condsLen = conds.length;

    this._view.stopIndicator();
    if (condsLen === 0) {
        this.stop();
        this.dispatchEvent(new Event(UPDATE));
        return;
    }

    this._setCond(e.args.filterId);
};

prototype._errorSetFilter = function(e) {
    var errorCode = e.args.errorCode;
    var errorDescription = _('auto_download_filter_add_filter_error');

    this._view.stopIndicator();
    switch(errorCode) {
        case 400:
        case 488:
        case 489:
        case 499:
            errorDescription = _(['auto_download_filter_add_filter_error_', errorCode].join(''));
            break;
    }
    this._errorDialog(errorDescription);
};

prototype._isMatchConds = function(cond1, cond2) {
    var isUndefined = function(vals) {
        for (var i = 0; i < vals.length; i += 1) {
            if (typeof vals[i] === 'undefined') return true;
        }
        return false;
    };

    if (isUndefined([cond1.name, cond1.op, cond1.value, cond2.name, cond2.op, cond2.value])) return false;
    if (cond1.name !== cond2.name) return false;
    if (cond1.op !== cond2.op) return false;
    if (cond1.value !== cond2.value) return false;

    if (! isUndefined([cond1.value2, cond2.value2])) {
        if (cond1.value2 !== cond2.value2) return false;
    }
    return true;
};

prototype._findCond = function(cond, conds) {
    var condsLen = conds.length;

    for (var i = 0; i < condsLen; i += 1) {
        if (this._isMatchConds(cond, conds[i])) return true;
    }
    return false;
};

prototype._diffConds = function(src, dst) {
    var list = [];
    var keys = Object.keys(src);
    var keysLen = keys.length;
    var s;
    var sLen;
    var d;

    for (var i = 0; i < keysLen; i += 1) {
        s = src[keys[i]];
        sLen = s.length;
        d = dst[keys[i]];
        for (var j = 0; j < sLen; j += 1) {
            if (!this._findCond(s[j], d)) list.push(s[j]);
        }
    }
    return list;
};

prototype._changeFilter = function() {
    var oldConds = this._oldFilterInfo.conds;
    var newConds = this._filterInfo.conds;
    var delList = this._diffConds(oldConds, newConds);
    var addList = this._diffConds(newConds, oldConds);
    var len;
    var query = {};

    this._view.startIndicator();
    this._requestingCondInfo.len = delList.length + addList.length;
    this._requestingCondInfo.error = false;

    if (this._oldFilterInfo.title !== this._filterInfo.title) {
        this._requestingCondInfo.len += 1;
    }
    if (this._oldFilterInfo.combCond !== this._filterInfo.combCond) {
        this._requestingCondInfo.len += 1;
    }

    if (this._oldFilterInfo.title !== this._filterInfo.title) {
        query.filterId = this._filterInfo.id;
        query.name = this._filterInfo.title;
        this._model.changeFilterName(query);
    }
    if (this._oldFilterInfo.combCond !== this._filterInfo.combCond) {
        query.filterId = this._filterInfo.id;
        query.combCond = this._filterInfo.combCond;
        this._model.changeFilterComb(query);
    }

    len = delList.length;
    for (var i = 0; i < len; i += 1) {
        query.condId = delList[i].condId;
        this._model.removeCond(query);
    }

    len = addList.length;
    for (var i = 0; i < len; i += 1) {
        query.filterId = this._filterInfo.id;
        query.name = addList[i].name;
        query.op = addList[i].op;
        query.value = addList[i].value;
        if (typeof addList[i].value2 !== 'undefined') query.value2 = addList[i].value2;
        this._model.setCond(query);
    }

    if (this._requestingCondInfo.len === 0) {
        this.stop();
        this.dispatchEvent(new Event(UPDATE));
    }
};

prototype._successChangeFilter = function() {
    this._requestingCondInfo.len -= 1;
    if (this._requestingCondInfo.len === 0) {
        this._completeChangeFilter();
    }
};

prototype._errorChangeFilter = function() {
    this._requestingCondInfo.len -= 1;
    this._requestingCondInfo.error = true;
    if (this._requestingCondInfo.len === 0) {
        this._completeChangeFilter();
    }
};

prototype._completeChangeFilter = function() {
    this._view.stopIndicator();
    if (this._requestingCondInfo.error) {
        this._errorDialog(_('auto_download_cond_error_description'));
        return;
    }
    this.stop();
    this.dispatchEvent(new Event(UPDATE));
};

prototype._extractCond = function(filterInfo) {
    var condsListLen = Utils.getObjectSize(this._filterInfo.conds);
    var conds = [];
    var len;

    for (var key in this._filterInfo.conds) {
        len = this._filterInfo.conds[key].length;
        if (len === 0) continue;
        for (var j = 0; j < len; j += 1) {
            conds.push(this._filterInfo.conds[key][j]);
        }
    }
    return conds;
};

prototype._setCond = function(filter_id) {
    var query = {};
    var conds = this._extractCond(this._filterInfo);
    var condsLen = conds.length;

    this._view.startIndicator();
    query.filterId = filter_id;
    this._requestingCondInfo.len = condsLen;
    this._requestingCondInfo.error = false;
    for (var i = 0; i < condsLen; i += 1) {
        query.name = conds[i].name;
        query.op = conds[i].op;
        query.value = conds[i].value;
        if (typeof conds[i].value2 !== 'undefined') query.value2 = conds[i].value2;
        this._model.setCond(query);
    }
};

prototype._clickSelectCondition = function(e) {
    var condId = e.args.condId;
    var query = {};
    var controller;

    query.condId = condId;
    query.conds = this._filterInfo.conds[condId];
    controller = new CondDialogController(query);
    controller.addEventListener(SETTING, this._clickSettingCondition, this);
    controller.addEventListener(NO_COND, this._clickNoCond, this);
    controller.start();
};

prototype._clickSetting = function(e) {
    this._filterInfo.title = e.args.title;
    this._filterInfo.combCond = e.args.combCond;

    if (this._filterInfo.title === '') {
        this._errorDialog(_('auto_download_filter_edit_filter_no_name_description'));
        return;
    }
    if (this._isEdit) {
        this._changeFilter();
    } else {
        this._setFilter();
    }
};

prototype._clickSettingCondition = function(e) {
    var conds = e.args.conds;
    var condsLen = conds.length;
    var name = e.args.name;

    this._delCondInFilterInfo(name);
    for (var i = 0, len = conds.length; i < len; i += 1) {
        this._filterInfo.conds[name].push(conds[i]);
    }
    this._view.setConditionStatus(name, (condsLen !== 0));
};

prototype._clickNoCond = function(e) {
    var condId = e.args.condId;

    this._delCondInFilterInfo(condId);
    this._view.setConditionStatus(condId, false);
};

prototype._errorDialog = function(text) {
    var query = {};
    var dialog;

    query.id = this._id;
    query.title = _('auto_download_filter_add_title');
    query.description = text;
    query.doubleDialog = true;
    dialog = new ErrorModalDialogView(query);
    dialog.show();
};

}());
