/**
 * @module dixim
 * @submodule autoDownload
 */
dixim.namespace('dixim.autoDownload');

/**
 * @module autoDownload
 * @submodule filter-setting-dialog-controller
 */

(function() {
var FilterSettingDialogController;
var prototype;
var superConstructor;
var FilterSettingDialogView = dixim.autoDownload.FilterSettingDialogView;
var EventTarget = dixim.event.EventTarget;
var _ = dixim.ctt.ResourceBundle.get;
var ApiClientAutoDownload = dixim.common.ApiClientAutoDownload;
var ErrorModalDialogView = dixim.common.ErrorModalDialogView;
var FilterEditDialogController = dixim.autoDownload.FilterEditDialogController;
var DeletionDialogController = dixim.autoDownload.DeletionDialogController;
var Utils = dixim.ctt.util.Utils;
var Enum = dixim.ctt.util.Enum;

/**
 * @class FilterSettingDialogController
 * @constructor
 * @param {Object} configs
 */
FilterSettingDialogController = function(configs) {
    EventTarget.call(this);
    this._initialize(configs);
};

dixim.inherit(FilterSettingDialogController, EventTarget);
dixim.autoDownload.FilterSettingDialogController = FilterSettingDialogController;
prototype = FilterSettingDialogController.prototype;
superConstructor = FilterSettingDialogController.superConstructor;

var SELECT = 'select';
var DELETE = 'delete';
var CHANGE = 'change';
var ADD = 'add';
var UPDATE = 'update';

var Mode = Enum.fetch('DELETION');
var FilterMode = Enum.fetch('DIALOG');

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
    configs = configs || {};

    configs.id = configs.id || '';
    this._id = [configs.id, 'filter-setting-dialog'].join('');

    this._model = new ApiClientAutoDownload();
    this._model.addEventListener('fetchFilterList', this._successFetchFilterList, this);
    this._model.addEventListener('fetchFilterListError', this._errorFetchFilterList, this);
    this._model.addEventListener('fetchCondList', this._successFetchCondList, this);
    this._model.addEventListener('fetchCondListError', this._errorFetchCondList, this);

    this._view = new FilterSettingDialogView(configs);
    this._view.addEventListener(SELECT, this._clickSelect, this);
    this._view.addEventListener(DELETE, this._clickDelete, this);
    this._view.addEventListener(CHANGE, this._clickChange, this);
    this._view.addEventListener(ADD, this._clickAdd, this);

    this._filterInfo = {};
    this._fetchCallback = null;
};

/**
 * @method start
 */
prototype.start = function() {
    this._view.show();
    this._view.updateActionBarStatus();

    this._model.fetchFilterList();
};

/**
 * @method stop
 */
prototype.stop = function() {
    this._view.hide();
};

prototype._successFetchFilterList = function(e) {
    var filters = e.args.filters;
    var filtersLen = filters.length;

    if (filtersLen === 0) {
        this._view.noFilter();
        return;
    }

    if (filtersLen > 10000) filtersLen = 10000;

    for (var i = 0; i < filtersLen; i += 1) {
        this._view.addFilter(filters[i]);
    }
    this._view.updateActionBarStatus();
};

prototype._errorFetchFilterList = function() {
    this._view.error(_('auto_download_filter_setting_fetch_error'));
};

prototype._successFetchCondList = function(e) {
    this._filterInfo.conds = e.args.conds;

    if (this._fetchCallback !== null) {
        this._fetchCallback();
        this._fetchCallback = null;
    }
};

prototype._errorFetchCondList = function(e) {
    this._errorDialog(_('auto_download_filter_setting_fetch_cond_error'));
};

prototype._fetchCondList = function(filter_id) {
    var query = {};

    query.filterId = filter_id;
    this._model.fetchCondList(query);
};

prototype._clickSelect = function() {
    this._view.switchSelect();
};

prototype._clickDelete = function(e) {
    var filters = e.args;
    var filtersLen = Utils.getObjectSize(filters);
    var controller;
    var query = {};

    if (filtersLen === 0) {
        this._errorDialog(_('auto_download_filter_error_no_selected'));
        return;
    }

    query.contents = [];
    for (var key in filters) {
        query.contents.push(filters[key]);
    }
    query.mode = Mode.FILTER;
    query.doubleDialog = true;
    controller = new DeletionDialogController(query);
    controller.addEventListener(UPDATE, this._update, this);
    controller.start();
};

prototype._clickChange = function(e) {
    var filters = e.args;
    var filtersLen = Utils.getObjectSize(filters);
    var callback = dixim.proxy(function() {
        var query = {};
        var controller;

        query.filterInfo = {
            id: this._filterInfo.filter.filterId,
            title: this._filterInfo.filter.filterName,
            combCond: this._filterInfo.filter.combCond,
        };
        query.conds = this._filterInfo.conds;
        query.mode = FilterMode.EDIT;
        controller = new FilterEditDialogController(query);
        controller.addEventListener(UPDATE, this._update, this);
        controller.start();
    }, this);

    if (filtersLen !== 1) {
        this._errorDialog(_('auto_download_filter_error_not_one'));
        return;
    }
    for (var key in filters) {
        this._filterInfo.filter = filters[key];
        break;
    }
    this._fetchCallback = callback;
    this._fetchCondList(this._filterInfo.filter.filterId);
};

prototype._clickAdd = function() {
    var controller;
    var query = {};

    query.mode = FilterMode.ADD;
    controller = new FilterEditDialogController(query);
    controller.addEventListener(UPDATE, this._update, this);
    controller.start();
};

prototype._update = function() {
    this._view.cleanList();
    this._view.updateActionBarStatus();
    this._model.fetchFilterList();
};

prototype._errorDialog = function(text) {
    var query = {};
    var dialog;

    query.id = this._id;
    query.title = _('auto_download_setting_filter_setting');
    query.description = text;
    query.doubleDialog = true;
    dialog = new ErrorModalDialogView(query);
    dialog.show();
};


}());
