/**
 * @module ctt
 * @submodule container-listing-controller
 */
(function() {
var Event = dixim.event.Event;
var EventTarget = dixim.event.EventTarget;
var ContainerListingView = dixim.ctt.ContainerListingView;
var ContentsTransferApiClient = dixim.ctt.api.ContentsTransferApiClient;

var ContainerListingController;
var prototype;

/**
 * Container list view controller.
 *
 * @class ContainerListingController
 * @param {Object} configs
 * @param {String} configs.container
 * @param {ContentsTransferApiClient} configs.model
 */
ContainerListingController = function(configs) {
  EventTarget.call(this);
  this._initialize(configs);
};
dixim.inherit(ContainerListingController, EventTarget);
dixim.ctt.ContainerListingController = ContainerListingController;
prototype = ContainerListingController.prototype;

/**
 * An event that is sent when the container is selected.
 * 
 * @event selectContainer
 * @param {Event} event an Event object that contains the following properties in its <em>args</em> property.
 * @param {Object} event.args
 * @param {Object} event.args.container a container data object that is selected.
 * @param {Object} event.args.device a device(such as storage, DMS, etc.) where the container is belong to.
 */
var SELECT_CONTAINER = 'selectContainer';

var REQUESTED_COUNT = 20;

var storageListToGroupList = function(data) {
  var groups = [];
  var storages = data.storages;
  var storage;
  var i;
  var len;

  len = storages.length;
  for (i = 0; i < len; ++i) {
    storage = storages[i];
    groups.push({
      id: storage.id,
      rootContainerId: storage.id,
      name: storage.name
    });
  }

  return groups;
};

var dmsListToGroupList = function(data) {
  var servers = data.results;
  var groups = [];
  var i;
  var len;
  var server;

  len = servers.length;
  for (i = 0; i < len; ++i) {
    server = servers[i];
    groups.push({
      id: server.udn,
      rootContainerId: '0',
      name: server.friendlyName
    });
  }

  return groups;
};

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 * @param {String} configs.containerNode
 * @param {ContentsTransferApiClient} configs.model
 */
prototype._initialize = function(configs) {
  this._view = new ContainerListingView({
    containerNode: configs.containerNode,
    selectionEnabled: true,
    embedded: configs.embedded
  });
  this._model = configs.model;
  this._isRemote = !!configs.isRemote;
  this._sessions = [];
  this._groupListSession = null;
  this._selectedContainer = null;
};

var _didSelectContainerImpl = function(group, container) {
  var event;
  var data;

  data = {
    container: container,
    device: group
  };
  this._selectedContainer = data;
  event = new Event(SELECT_CONTAINER, data);
  this.dispatchEvent(event);
};

/**
 * @method _didSelectContainer
 * @protected
 */
prototype._didSelectContainer = function(e) {
  _didSelectContainerImpl.call(this, e.args.group, e.args.container);
};

/**
 * @method _didSelectServer
 * @protected
 */
prototype._didSelectServer = function(e) {
  var containerId;

  if (this._isRemote) {
    containerId = '0';
  } else {
    containerId = e.args.group.rootContainerId;
  }
  _didSelectContainerImpl.call(this, e.args.group, {id: containerId});
};

/**
 * At this moment, this method just kicks the group list loading.
 *
 * @method _didRenderView
 * @protected
 */
prototype._didRenderView = function(e) {
  this._fetchGroupList();
};

/**
 * @method _fetchGroupList
 * @protected
 */
prototype._fetchGroupList = function() {
  if (!!this._groupListSession) return;

  if (this._isRemote) {
    this._groupListSession = this._model.dnDmsList();
  } else {
    this._groupListSession = this._model.storageList();
  }
};

/**
 * @method _didFetchGroupList
 * @protected
 */
prototype._didFetchGroupList = function(e) {
  var args = e.args;
  var session = args.session;
  var data = args.data;

  if (this._groupListSession !== session) {
    return;
  }
  if (!session.isCancelled()) {
    this._view.updateGroups(this._toGroupList(data));
  }
  this._groupListSession = null;
};

prototype._toGroupList = function(data) {
  var toGroupList;

  if (this._isRemote) {
    toGroupList = dmsListToGroupList;
  } else {
    toGroupList = storageListToGroupList;
  }
  return toGroupList(data);
};

prototype._isBeingRequested = function(device, container, startingIndex) {
  var found = false;

  dixim.array.each(this._sessions, function(i, session) {
    var ud;

    if (!(ud = session.userData)) return;
    if (ud.device.id === device.id &&
        ud.container.id === container.id &&
        ud.startingIndex == startingIndex) {
      found = true;
      return false;
    }
  });

  return found;
};

/**
 * @method _didAddGroup
 * @protected
 */
prototype._didAddGroup = function(e) {
  this._browseContainer(e.args.group);
};

/**
 * @method _didOpenContainer
 * @protected
 */
prototype._didOpenContainer = function(e) {
  this._browseContainer(e.args.group, e.args.container);
};

/**
 * @method _didCloseContainer
 * @protected
 */
prototype._didCloseContainer = function(e) {
  this._cancelBrowseSessions(e.args.group, e.args.container);
};

prototype._cancelBrowseSessions = function(device, container) {
  dixim.array.each(this._sessions, function(i, session) {
    var ud;

    if (!(ud = session.userData)) return;
    if (ud.device.id === device.id &&
        ud.container.id === container.id) {
      session.cancel();
    }
  });
};

/**
 * @method _browseContainer
 * @protected
 */
prototype._browseContainer = function(device, container, startingIndex) {
  var session;
  var params;

  container = container || {id: device.rootContainerId};
  startingIndex = startingIndex || 0;

  if (this._isBeingRequested(device, container, startingIndex)) {
    return;
  }

  params = {
    startingIndex: startingIndex,
    requestedCount: REQUESTED_COUNT,
    id: container.id
  };

  if (this._isRemote) {
    params.udn = device.id;
    session = this._model.dnBrowse(params);
  } else {
    params.filter = 'container';
    session = this._model.browse(params);
  }

  session.userData = {
    container: container,
    device: device
  };
  this._sessions.push(session);
};

var extractContainers = function(didls) {
  var len = didls.length;
  var i;
  var didl;
  var containers = [];

  for (i = 0; i < len; i += 1) {
    didl = didls[i];
    if (didl['class'] === 'container') {
      containers.push(didl);
    }
  }

  return containers;
};

var calculateNextStartingIndex = function(data) {
  var index;
  var total = data.totalMatches || 0;

  if (data.numberReturned === 0) return total;
  index = data.startingIndex + data.numberReturned;
  if (index < total) {
    return index;
  }
  return total;
};

/**
 * @method _didBrowseContainer
 * @protected
 */
prototype._didBrowseContainer = function(e) {
  var args = e.args;
  var session = args.session;
  var sessions = this._sessions;
  var userData = session.userData;
  var data = args.data;
  var results = data.results;
  var children;
  var nextStartingIndex;
  var total = data.totalMatches || 0;

  if (dixim.array.findIndex(sessions, session) === sessions.length) {
    return;
  }

  if (!session.isCancelled()) {
    children = extractContainers(results);
    this._view.addChildren({
      parentId: args.parentId,
      children: children,
      groupId: userData.device.id
    });

    if (children.length === results.length) {
      nextStartingIndex = calculateNextStartingIndex(data);
      if (nextStartingIndex < total) {
        this._browseContainer(userData.device, userData.container, nextStartingIndex);
      }
    }
  }

  dixim.array.remove(sessions, session);
};

/**
 * @method _didFailBrowseContainer
 * @protected
 */
prototype._didFailBrowseContainer = function(e) {
  var session = e.args.session;
  var ud = session.userData;

  dixim.array.remove(this._sessions, session);
  if (!!!ud) return;
  this._view.setBrowseError(ud.device, ud.container);
};

/**
 * @method _didChangePin
 * @protected
 */
prototype._didChangePin = function(e) {
  this._reload();
};

/**
 * @method _cancelAllSessions
 * @protected
 */
prototype._cancelAllSessions = function() {
  if (!!this._groupListSession) {
    this._groupListSession.cancel();
  }

  dixim.array.each(this._sessions, function(i, session) {
    session.cancel();
  });
};

/**
 * @method cancelAllSessions
 */
prototype.cancelAllSessions = function() {
  this._cancelAllSessions();
};

/**
 * @method _setupEventHandlers
 * @protected
 */
prototype._setupEventHandlers = function() {
  this._view.addEventListener('render', this._didRenderView, this);
  this._view.addEventListener('selectServer', this._didSelectServer, this);
  this._view.addEventListener('selectContainer', this._didSelectContainer, this);
  this._view.addEventListener('openContainer', this._didOpenContainer, this);
  this._view.addEventListener('closeContainer', this._didCloseContainer, this);
  this._view.addEventListener('addGroup', this._didAddGroup, this);

  if (this._isRemote) {
    this._model.addEventListener('dnDmsList', this._didFetchGroupList, this);
    this._model.addEventListener('dnBrowse', this._didBrowseContainer, this);
    this._model.addEventListener('dnBrowseError', this._didFailBrowseContainer, this);
  } else {
    this._model.addEventListener('storageList', this._didFetchGroupList, this);
    this._model.addEventListener('browse', this._didBrowseContainer, this);
    this._model.addEventListener('browseError', this._didFailBrowseContainer, this);
  }
  this._model.addEventListener('changePin', this._didChangePin, this);
};

/**
 * @method start
 */
prototype.start = function() {
  this._setupEventHandlers();
  this._view.show();
};

/**
 * Sets PIN.
 *
 * @method setPin
 * @param {String} pin
 */
prototype.setPin = function(pin) {
  this._model.setPin(pin);
};

/**
 * Reloads container list
 *
 * @method reload
 */
prototype.reload = function() {
  this._reload();
};

prototype._reload = function() {
  this._cancelAllSessions();
  this._fetchGroupList();
};

/**
 * @method getSelectedContainer
 * @return {Object} an object has field like {container: ..., device: ...}.
 */
prototype.getSelectedContainer = function() {
  return this._selectedContainer;
};

}());
