/**
 * @module ctt
 * @submodule invasion-detector
 */
(function() {
var Event = dixim.event.Event;
var EventTarget = dixim.event.EventTarget;
var Y = dixim.ctt.YUI;

var InvasionDetector;

/**
 * @class InvasionDetector
 * @constructor
 * @param {Object} configs
 * @param {Node} configs.region
 * @param {Node} configs.invader
 * @param {Number} [configs.interval=100] detection interval (msec)
 */
InvasionDetector = function(configs) {
  EventTarget.call(this);
  this._initialize(configs);
};
dixim.inherit(InvasionDetector, EventTarget);
dixim.ctt.InvasionDetector = InvasionDetector;
prototype = InvasionDetector.prototype;

/**
 * @event hasCome
 */
var HAS_COME = 'hasCome';

/**
 * @event hasGone
 */
var HAS_GONE = 'hasGone';

prototype._initialize = function(configs) {
  this._region = configs.region;
  this._invader = configs.invader;
  this._interval = configs.interval || 100;
};

prototype._onTimeToScan = function() {
  var args;

  if (!!!this._intervalId) return;

  args = {
    detector: this,
    region: this._region,
    invader: this._invader
  };
  if (this._invader.inRegion(this._region)) {
    this.dispatchEvent(new Event(HAS_COME, args));
  } else {
    this.dispatchEvent(new Event(HAS_GONE, args));
  }
};

prototype.start = function() {
  if (!!this._intervalId) return;
  this._intervalId = setInterval(dixim.proxy(function() {
    this._onTimeToScan();
  },this), this._interval);
};

prototype._invalidateInterval = function() {
  if (!!!this._intervalId) return;
  clearInterval(this._intervalId);
  this._intervalId = null;
};

prototype.stop = function() {
  this._invalidateInterval();
};

prototype.invalidate = function() {
  this._invalidateInterval();
};

}());

/**
 * @module ctt
 * @submodule content-list-table-view
 */
(function() {
var Event = dixim.event.Event;
var EventTarget = dixim.event.EventTarget;
var ContentListTableViewFormatter = dixim.ctt.ContentListTableViewFormatter;
var InvasionDetector = dixim.ctt.InvasionDetector;
var Configs = dixim.ctt.Configs;
var encodeId = dixim.xml.encodeId;
var decodeId = dixim.xml.decodeId;
var _ = dixim.ctt.ResourceBundle.get;
var Y = dixim.ctt.YUI;
var $ = dixim.ctt.jQuery;

var ContentListTableView;
var prototype;

/**
 * 
 * @class ContentListTableView
 * @constructor
 * @uses YUI
 * @param {Object} configs
 * @param {String} configs.containerNode
 * @param {Boolean} configs.isRemote
 */
ContentListTableView = function(configs) {
  EventTarget.call(this);
  this._initialize(configs);
};
dixim.inherit(ContentListTableView, EventTarget);
dixim.ctt.ContentListTableView = ContentListTableView;
prototype = ContentListTableView.prototype;

var ELEMENT_ID_PREFIX = 'content-list-table-view-';
var STATIC_ROW_ID = ELEMENT_ID_PREFIX + 'static-row';
var CLASS_SORTABLE_COLUMN = 'content-list-table-view-sortable-column';
var CLASS_SORTABLE_COLUMN_ASC = 'content-list-table-view-sortable-column-asc';
var CLASS_SORTABLE_COLUMN_DES = 'content-list-table-view-sortable-column-des';
var CLASS_ROW_SELECTED = 'content-list-table-view-row-selected';

var TEMPLATE_TABLE_CONTAINER = '<div class="yui3-skin-sam content-list-table-view"></div>';

var TEMPLATE_SORTABLE_COLUMN_HEADER_CELL = [
  '<th id="{id}" colspan="{_colspan}" rowspan="{_rowspan}"',
  ' class="{className} ' + CLASS_SORTABLE_COLUMN + '"',
  ' scope="col" {_id}{abbr}>',
  '<input type="hidden" name="columnName" value="${columnName}" />',
  '{content}',
  '<img class="content-list-table-view-sortable-column-icon-asc" src="', Configs.get('imagePath'), '/sort_asc.png' ,'" width="16" height="8" alt="^" />',
  '<img class="content-list-table-view-sortable-column-icon-des" src="', Configs.get('imagePath'), '/sort_des.png' ,'" width="16" height="8" alt="v" />',
  '</th>'
].join('');

var STATIC_ROW = {
  id: STATIC_ROW_ID,
  title: _('ctt_table_static_row_title_back'),
  "class": "parentContainer",
  isStaticRow: true
};

// XXX: This column definition may be modified by _getTableColumnDefinitions() dynamically.
//      See the method for details.
var COLUMNS = (function(f) {
  var ht = TEMPLATE_SORTABLE_COLUMN_HEADER_CELL;

  return [
    {
      key: 'rowSelector',
      name: 'rowSelector',
      label: ' ',
      allowHTML: true,
      resizable: false,
      formatter: f.rowSelector
    },
    {
      key: 'title',
      name: 'title',
      headerTemplate: ht.replace('${columnName}', 'title'),
      label: _('ctt_table_column_label_title'),
      sortable: false,
      allowHTML: true,
      resizable: false,
      formatter: f.title
    },
    {
      key: 'compressState',
      name: 'compressState',
      label: _('ctt_table_column_label_compress_state'),
      sortable: false,
      resizable: false,
      allowHTML: true,
      formatter: f.compressState
    },
    {
      key: 'date',
      name: 'date',
      headerTemplate: ht.replace('${columnName}', 'date'),
      label: _('ctt_table_column_label_date'),
      sortable: false,
      resizable: false,
      allowHTML: true,
      formatter: f.date
    },
    {
      key: 'size',
      name: 'size',
      label: _('ctt_table_column_label_size'),
      sortable: false,
      resizable: false,
      allowHTML: true,
      formatter: f.size
    },
    {
      key: 'tags',
      name: 'tags',
      id: ELEMENT_ID_PREFIX + 'tags',
      label: ' ',
      allowHTML: true,
      resizable: false,
      formatter: f.tags
    }
  ];
}(ContentListTableViewFormatter));

/**
 * Triggered when the container has been rendered.
 * 
 * @event render
 */
var RENDER = 'render';

/**
 * Triggered when the list has
 * been scrolled to the end of the list.
 */
var SCROLL_TO_THE_END = 'scrollToTheEnd';

/**
 * Triggered when the item selection has been changed.
 */
var CHANGE_SELECTION = 'changeSelection';

/**
 * Triggered when an item has been clicked.
 */
var CLICK_ITEM = 'clickItem';

/**
 * Triggered when a sortable column header was clicked.
 */
var REQUEST_CHANGE_SORT_CRITERIA = 'requestChangeSortCriteria';


// converts YUI.DataTable record data to DIDL object.
var recordToDidl = function(record) {
  return {
    id: record.get('id'),
    parentId: record.get('parentId'),
    title: record.get('title'),
    'class': record.get('class'),
    mediaType: record.get('mediaType'),
    date: record.get('date'),
    url: record.get('url'),
    profile: record.get('profile'),
    size: record.get('size'),
    removable: record.get('removable'),
    uploadable: record.get('uploadable'),
    parentalLock: record.get('parentalLock'),
    copyCount: record.get('copyCount'),
    dn_movable: record.get('dn_movable'),
    dnMovable: record.get('dn_movable'),
    autoRemoveProtected: record.get('autoRemoveProtected'),
    writeStatus: record.get('writeStatus'),
    atsTranscodeState: record.get('atsTranscodeState')
  };
};

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 * @param {String} configs.containerNode
 * @param {Boolean} configs.isRemote
 */
prototype._initialize = function(configs) {
  var node;

  node = configs.containerNode;
  // XXX: Purifying the containerNode.
  //      Actually, the containerNode can be a jQuery object.
  //      However, this class uses YUI as its main JS library.
  //      Thus, if the containerNode is a jQuery object,
  //      it has to be converted to either an ID selector string
  //      or a naked Element object.
  if (typeof node.jquery !== 'undefined') {
    if (!!(node = node.attr('id'))) {
      node = '#' + node;
    } else {
      node = node.get(0);
    }
  }
  // XXX: ...Purifying ended.
  this._containerNode = node;
  this._isRemote = !!configs.isRemote;
  this._isAutoDelete = !!configs.isAutoDelete;
  this._isCompression = !!configs.isCompression;
  ContentListTableViewFormatter._isAutoDelete = !!this._isAutoDelete;
  this._view = null;
  this._invasionDetector = null;
  this._dataTable = null;
  this._scrollContainer = null;
  this._sortData = null;
  this._selectedRows = {};
};

/**
 * @method show
 */
prototype.show = function() {
  if (!!!this._view) {
    this._createView();
    this._dataTable.render(this._view);
  }
  this._dataTable.show();
  this._view.show();
};

/**
 * @method _createView
 * @private
 */
prototype._createView = function() {
  var container;
  var v;

  container = Y.one(this._containerNode);
  v = Y.Node.create(TEMPLATE_TABLE_CONTAINER);
  v.setStyles({width: '100%', height: '100%'});
  v.appendTo(container);

  this._view = v;
  this._createDataTable();
};

prototype._getTableColumnDefinitions = function() {
  var column;
  var columnDefinitions = [];
  var len = COLUMNS.length;
  var i;

  if (!this._isRemote) return COLUMNS;

  for (i = 0; i < len; i += 1) {
    column = dixim.extend({}, COLUMNS[i], true);
    // XXX: Since dn_browse API doesn't support sorting,
    //      all columns must be un-sortable when
    //      browsing remote servers. (i.e. this._isRemote is true)
    if (!!column.headerTemplate) {
      delete column.headerTemplate;
    }
    if (!this._isCompression && column.key === "compressState") {
      continue;
    }
    // XXX: ...end.
    if (column.key === 'rowSelector') {
      column.formatter = ContentListTableViewFormatter.rowSelectorForDownloadMoveIn;
    }
    columnDefinitions.push(column);
  }

  return columnDefinitions;
};

prototype._createDataTable = function() {
  var container;
  var dt;
  var columns;

  columns = this._getTableColumnDefinitions();
  container = this._view;
  dt = new Y.DataTable({
    columns: columns,
    data: [],
    scrollable: "y",
    width: container.getComputedStyle('width'),
    height: container.getComputedStyle('height')
  });

  dt.onceAfter('datatable:render', this._didRenderView, this);
  dt.delegate('click', this._didClickRowSelector, 'input[name="rowSelector"]', this);
  dt.delegate('click', this._didClickTitleCell, '.content-list-table-view-cell-title', this);
  dt.delegate('click', this._didClickSortableColumnHeader, 'th.' + CLASS_SORTABLE_COLUMN, this);
  this._dataTable = dt;
};

prototype._didRenderView = function(e) {
  this._scrollContainer = this._view.one('.yui3-datatable-y-scroller-container');
  this.dispatchEvent(new Event(RENDER));
  this._updateInvasionDetector();
};

prototype._didClickTitleCell = function(e) {
  var target = e.target;
  var dt = this._dataTable;
  var args = {};
  var id;
  var parent;

  parent = target.get('parentNode');
  id = decodeId(parent.one('input[type=hidden][name=recordId]').get('value'));
  args.data = recordToDidl(dt.getRecord(id));
  this.dispatchEvent(new Event(CLICK_ITEM, args));
};

prototype._didClickRowSelector = function(e) {
  var target = e.target;
  var id;

  id = decodeId(target.get('value'));
  if (target.get('checked')) {
    this._selectRow(id);
  } else {
    this._unselectRow(id);
  }
  this.dispatchEvent(new Event(CHANGE_SELECTION));
};

prototype.setSortCriteria = function(criteria) {
  var data;
  var order;
  var node;
  var key;

  this._clearSortStatus();
  if (!!!criteria || !criteria.length) return;

  key = criteria[0].key;
  order = criteria[0].order;

  // XXX: It seems that there is no column header cell getter,
  //      so traverses contentBox instead.
  node = this._view.one([
    '.yui3-datatable-header',
    '.yui3-datatable-col-', key,
    '.content-list-table-view-sortable-column'
  ].join(''));
  // XXX: ...end.
  if (!!!node) return;

  if (order) {
    node.addClass(CLASS_SORTABLE_COLUMN_DES);
  } else {
    node.addClass(CLASS_SORTABLE_COLUMN_ASC);
  }

  data = {};
  data.node = node;
  data.order = order;
  data.key = key;
  this._sortData = data;
};

prototype._didClickSortableColumnHeader = function(e) {
  var target = e.currentTarget;
  var curentId = null;
  var targetId = null;
  var data;
  var order;
  var args;
  var selector = 'input[type=hidden][name=columnName]';

  if (!!!(targetId = target.one(selector).get('value'))) return;
  if (!!(data = this._sortData)) {
    curentId = data.key;
  }

  if (!!curentId && targetId === curentId) {
    order = !data.order;
  } else {
    order = false;
  }

  args = {criteria: [{key: targetId, order: order}]};
  this.dispatchEvent(new Event(REQUEST_CHANGE_SORT_CRITERIA, args));
};

prototype._clearSortStatus = function() {
  var node;

  if (!!!this._sortData) {
    return;
  }

  node = this._sortData.node;
  node.removeClass(CLASS_SORTABLE_COLUMN_ASC);
  node.removeClass(CLASS_SORTABLE_COLUMN_DES);
  this._sortData = null;
};

prototype._selectRow = function(id) {
  var dt = this._dataTable;
  var row;

  row = dt.getRow(dt.getRecord(id));
  row.addClass(CLASS_ROW_SELECTED);
  if (row.one(['input[name="rowSelector"][value="' + encodeId(id) + '"]'].join('')) !== null) {
    row.one(['input[name="rowSelector"][value="' + encodeId(id) + '"]'].join('')).set('checked', true);
  }
  this._selectedRows[id] = true;
};

prototype._unselectRow = function(id) {
  var dt = this._dataTable;
  var row;

  row = dt.getRow(dt.getRecord(id));
  row.removeClass(CLASS_ROW_SELECTED);
  if (row.one(['input[name="rowSelector"][value="' + encodeId(id) + '"]'].join('')) !== null) {
    row.one(['input[name="rowSelector"][value="' + encodeId(id) + '"]'].join('')).set('checked', false);
  }
  if (typeof this._selectedRows[id] !== undefined) {
    delete this._selectedRows[id];
  }
};

prototype.selectAllRows = function() {
  var dt = this._dataTable;
  var len;
  var i;
  var records;
  var record;
  var id;

  records = dt.data.toArray();
  len = records.length;
  for (i = 0; i < len; i += 1) {
    record = records[i];
    if (record.get('isStaticRow')) continue;
    if (!record.get('dn_movable')) continue;
    id = record.get('id');
    this._selectRow(id);
  }
  this.dispatchEvent(new Event(CHANGE_SELECTION));
};

prototype.unselectAllRows = function() {
  this._unselectAllRows();
};

prototype._unselectAllRows = function() {
  var dt = this._dataTable;
  var len;
  var i;
  var records;
  var record;
  var rows = this._selectedRows;
  var id;

  records = dt.data.toArray();
  len = records.length;

  for (i = 0; i < len; i += 1) {
    record = records[i];
    if (record.get('isStaticRow')) continue;
    if (!record.get('dn_movable')) continue;
    id = record.get('id');
    if (typeof rows[id] !== 'undefined') {
      this._unselectRow(id);
    }
  }
  this._selectedRows = {};
  this.dispatchEvent(new Event(CHANGE_SELECTION));
};

/**
 * @method hide
 */
prototype.hide = function() {
  this._dataTable.hide();
  this._view.hide();
};

/**
 * @method destroy
 */
prototype.destroy = function() {
  this._view.hide();
  this._dataTable.destroy();
  this._view = null;
};

prototype.enableStaticRow = function() {
  this._enableStaticRow();
};

prototype._enableStaticRow = function() {
  var record;
  var dt = this._dataTable;
  var data = dt.data;
  var list;

  record = dt.getRecord(STATIC_ROW_ID);
  if (!!record) return;

  list = data.toArray();
  list = [STATIC_ROW].concat(list);
  dt.set('data', list);
};

prototype.disableStaticRow = function() {
  this._disableStaticRow();
};

prototype._disableStaticRow = function() {
  this._dataTable.removeRow(STATIC_ROW_ID);
};

prototype._isStaticRowEnabled = function() {
  return !!(this._dataTable.getRecord(STATIC_ROW_ID));
};

prototype.getNumberOfRows = function(records) {
  if (this._isStaticRowEnabled()) {
    return this._dataTable.data.size() - 1;
  } else {
    return this._dataTable.data.size();
  }
};

prototype.addRows = function(records) {
  var dt = this._dataTable;
  var checked_contents = [];

  $('.yui3-datatable-table input:checked').each(function() {
    checked_contents.push($(this).val());
  });
  if (!records.length) return;
  dt.addRows(records, {silent: true});
  dt.syncUI();
  // XXX: share logics that are executed after data has been changed.
  this._unselectAllRows();
  this._updateInvasionDetector();
  // XXX: ...end
  $('.yui3-datatable-table input').each(function() {
    var value = $(this).val();
    var checked_contents_length = checked_contents.length;

    for(var i = 0; i < checked_contents_length; i += 1) {
      if (value === checked_contents[i]) {
        $(this).trigger('click');
      }
    }
  });
};

prototype.removeAllRows = function() {
  this._dataTable.set('data', []);
  // XXX: share logics that are executed after data has been changed.
  this._unselectAllRows();
  this._updateInvasionDetector();
  // XXX: ...end
};

prototype.getSelectedRows = function() {
  var rows = this._selectedRows;
  var dt = this._dataTable;
  var ret = [];
  var id;

  for (id in rows) {
    if (!rows.hasOwnProperty(id)) continue;
    ret.push(recordToDidl(dt.getRecord(id)));
  }
  return ret;
};

prototype._updateInvasionDetector = function() {
  var detector = this._invasionDetector;
  var dt = this._dataTable;
  var invader;
  var size;

  if (!!detector) {
    detector.invalidate();
    this._invasionDetector = null;
  }

  if ((size = dt.data.size())) {
    invader = dt.getRow(size - 1);
  } else {
    invader = this._scrollContainer;
  }

  detector = new InvasionDetector({
    region: this._scrollContainer,
    invader: invader,
    interval: 500
  });
  detector.addEventListener('hasCome', this._invaderHasCome, this);
  detector.start();
  this._invasionDetector = detector;
};

prototype._invaderHasCome = function() {
  this._invasionDetector.stop();
  this.dispatchEvent(new Event(SCROLL_TO_THE_END));
  this._invasionDetector.start();
};

}());
