/**
 * @module dixim
 * @submodule common
 */
dixim.namespace('dixim.common');

/**
 * @module common
 * @submodule api-client-vdms
 */
(function() {
var ApiClientVdms;
var prototype;
var suprtConstructor;
var ApiClient = dixim.common.ApiClient;
var Configs = dixim.ctt.Configs;

/**
 * Provides interfaces to setting vdms server API.
 *
 * This API client supports two type of event handling method,
 * callback and event-subscription.
 *
 * The callback method can be specified as the parameter of each method.
 * (Such as request.success, request.error or request.complete.)
 * When the callback is called, an Event instance which is related to
 * the triggered method is passed to the callback.
 *
 * This API client is also an EventTarget, you can subscribe an event
 * by addEventListener().
 *
 * @class ApiClientVdms
 * @constructor
 * @extends EventTarget
 * @param {Object} [configs]
 */
ApiClientVdms = function(configs) {
    ApiClient.call(this, configs);
};

dixim.inherit(ApiClientVdms, ApiClient);
dixim.common.ApiClientVdms = ApiClientVdms;
prototype = ApiClientVdms.prototype;
superConstructor = ApiClientVdms.suprtConstructor;

/**
 * Fetch allow list.
 *
 * @method fetchAllowList
 * @param {Object} request
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.fetchAllowList = function(request) {
    return this.doSimpleFetch.call(this, 'fetchAllowList', [Configs.get('apiAcsVdms'), '/GetAllowList'].join(''), request, 'POST');
};

/**
 * Fetch deny list.
 *
 * @method fetchDenyList
 * @param {Object} request
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.fetchDenyList = function(request) {
    return this.doSimpleFetch.call(this, 'fetchDenyList', [Configs.get('apiAcsVdms'), '/GetDenyList'].join(''), request, 'POST');
};

/**
 * Set permission.
 *
 * @method setPermission
 * @param {Object} request
 * @param {Array} [request.MacList]
 * @param {String} [request.Permission]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.setPermission = function(request) {
    var query = {};

    query.MacList = request.MacList.join(',');
    query.Permission = request.Permission;

    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'setPermission', [Configs.get('apiAcsVdms'), '/SetPermission'].join(''), query, request, 'POST');
};

/**
 * remove entries.
 *
 * @method removeEntries
 * @param {Object} request
 * @param {Array} [request.Entries]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.removeEntries = function(request) {
    var query = {};

    query.Entries = request.Entries.join(',');

    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'removeEntries', [Configs.get('apiAcsVdms'), '/RemoveEntries'].join(''), query, request, 'POST');
};

/**
 * get default Permission.
 *
 * @method getDefaultPermission
 * @param {Object} request
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.getDefaultPermission = function(request) {
    return this.doSimpleFetch.call(this, 'getDefaultPermission', [Configs.get('apiAcsVdms'), '/GetDefaultPermission'].join(''), request);
};

/**
 * set default Permission.
 *
 * @method setDefaultPermission
 * @param {Object} request
 * @param {String} [request.DefaultPermission]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.setDefaultPermission = function(request) {
    var query = {};

    query.DefaultPermission = request.DefaultPermission;

    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'setDefaultPermission', [Configs.get('apiAcsVdms'), '/SetDefaultPermission'].join(''), query, request);
};

/**
 * Fetches dms status.
 *
 * @method fetchDmsStatus
 * @param {Object} request
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.fetchDmsStatus = function(request) {
    return this.doSimpleFetch.call(this, 'fetchDmsStatus', [Configs.get('apiDeviceManagerVdms'), '/status/get'].join(''), request);
};

/**
 * Change dms status.
 *
 * @method fetchDmsStatus
 * @param {Object} request
 * @param {String} [request.status]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.changeDmsStatus = function(request) {
    var query = {};

    query.status = request.status;

    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'changeDmsStatus', [Configs.get('apiDeviceManagerVdms'), '/status/mod'].join(''), query, request);
};

/**
 * Fetch pairing devices.
 *
 * @method fetchPairingDevice
 * @param {Object} request
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.fetchPairingDevice = function(request) {
    return this.doSimpleFetch.call(this, 'fetchPairingDevice', [Configs.get('apiDragdPlus'), '/device/get'].join(''), request, 'POST');
};

/**
 * Remove pairing devices.
 *
 * @method removePairingDevice
 * @param {Object} request
 * @param {String} [request.deviceId]
 * @param {String} [request.dtlaIdHash]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.removePairingDevice = function(request) {
    var query = {};

    query.deviceId = request.deviceId;
    query.dtlaIdHash = request.dtlaIdHash;

    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'removePairingDevice', [Configs.get('apiDragdPlus'), '/device/del'].join(''), query, request);
};

/**
 * Fetches dtcp-ip activation status.
 *
 * @method fetchDas
 * @param {Object} request
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.fetchDas = function(request) {
    return this.doSimpleFetch.call(this, 'fetchDas', [Configs.get('apiDasVdms'), '/status/get'].join(''), request);
};

/**
 * Activate dtcp-ip auth.
 *
 * @method activateDas
 * @param {Object} request
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.activateDas = function(request) {
    return this.doSimpleFetch.call(this, 'activateDas', [Configs.get('apiDasVdms'), '/activate'].join(''), request);
};

/**
 * Fetches aggregation server.
 *
 * @method fetchAggregationList
 * @param {Object} request
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.fetchAggregationList = function(request) {
    return this.doSimpleFetch.call(this, 'fetchAggregationList', [Configs.get('apiCcsVdms'), '/dms/enable_list'].join(''), request);
};

/**
 * Fetches non aggregation server.
 *
 * @method fetchNonAggregationList
 * @param {Object} request
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.fetchNonAggregationList = function(request) {
    return this.doSimpleFetch.call(this, 'fetchNonAggregationList', [Configs.get('apiCcsVdms'), '/dms/disable_list'].join(''), request);
};

/**
 * Enable aggregation server.
 *
 * @method enableAggregation
 * @param {Object} request
 * @param {String} [request.udn]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.enableAggregation = function(request) {
    var query = {};

    query.udn = request.udn;

    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'enableAggregation', [Configs.get('apiCcsVdms'), '/dms/enable'].join(''), query, request, 'POST');
};

/**
 * Disable aggregation server.
 *
 * @method disableAggregation
 * @param {Object} request
 * @param {String} [request.udn]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.disableAggregation = function(request) {
    var query = {};

    query.udn = request.udn;

    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'disableAggregation', [Configs.get('apiCcsVdms'), '/dms/disable'].join(''), query, request);
};

/**
 * Remove aggregation server.
 *
 * @method removeAggregation
 * @param {Object} request
 * @param {String} [request.udn]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.removeAggregation = function(request) {
    var query = {};

    query.udn = request.udn;

    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'removeAggregation', [Configs.get('apiCcsVdms'), '/dms/remove'].join(''), query, request);
};

/**
 * Sync aggregation server.
 *
 * @method syncAggregation
 * @param {Object} request
 * @param {String} [request.udn]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.syncAggregation = function(request) {
    var query = {};

    query.udn = request.udn;

    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'syncAggregation', [Configs.get('apiCcsVdms'), '/dms/sync'].join(''), query, request);
};

/**
 * Cancel sync aggregation server.
 *
 * @method cancelSyncAggregation
 * @param {Object} request
 * @param {String} [request.udn]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.cancelSyncAggregation = function(request) {
    var query = {};

    query.udn = request.udn;

    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'cancelSyncAggregation', [Configs.get('apiCcsVdms'), '/dms/cancel_sync'].join(''), query, request);
};

/**
 * Fetches aggregation default setting 
 *
 * @method fetchDefaultAggregation
 * @param {Object} request
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.fetchDefaultAggregation = function(request) {
    return this.doSimpleFetch.call(this, 'fetchDefaultAggregation', [Configs.get('apiCcsVdms'), '/default_crawl/get'].join(''), request);
};

/**
 * Set aggregation default setting.
 *
 * @method setDefaultAggregation
 * @param {Object} request
 * @param {Integer} [request.defaultCrawl]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.setDefaultAggregation = function(request) {
    var query = {};

    query.defaultCrawl = request.defaultCrawl;

    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'setDefaultAggregation', [Configs.get('apiCcsVdms'), '/default_crawl/set'].join(''), query, request);
};

/**
 * Fetches proxy setting 
 *
 * @method fetchProxyMode
 * @param {Object} request
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.fetchProxyMode = function(request) {
    return this.doSimpleFetch.call(this, 'fetchProxyMode', [Configs.get('apiPdsVdms'), '/proxy_mode/get'].join(''), request);
};

/**
 * Set proxy setting.
 *
 * @method setProxyMode
 * @param {Object} request
 * @param {Integer} [request.proxyMode]
 * @param {Function} [request.success]
 * @param {Function} [request.error]
 * @param {Function} [request.complete]
 * @return {ApiClientSession} session
 */
prototype.setProxyMode = function(request) {
    var query = {};

    query.proxyMode = request.proxyMode;

    request.dataType = 'text';

    return this.doSimpleMod.call(this, 'setProxyMode', [Configs.get('apiPdsVdms'), '/proxy_mode/set'].join(''), query, request);
};



}());
